import os
import re
import json
import requests
from urllib import parse

import BiliUtil.static_value as v
import BiliUtil.static_func as f
from BiliUtil.bili_video import Video


class Album:
    cookie = None

    aid = None

    name = None
    time = None
    desc = None
    zone = None
    num = None
    cover = None
    like = None
    coin = None
    favorite = None
    share = None
    view = None
    danmu = None
    video_list = list()

    def __init__(self, aid=None):
        self.aid = aid

    def set_album(self, aid=None):
        self.aid = aid
        self.name = None
        self.time = None
        self.desc = None
        self.zone = None
        self.num = None
        self.cover = None
        self.like = None
        self.coin = None
        self.favorite = None
        self.share = None
        self.view = None
        self.danmu = None
        self.video_list = list()

    def set_by_url(self, url):
        input_url = parse.urlparse(url)
        aid = re.match('/video/av([0-9]+)', input_url.path).group(1)
        self.set_album(aid)

    def set_cookie(self, cookie):
        self.cookie = cookie
        for video in self.video_list:
            video.set_cookie(cookie)

    def get_album_info(self):
        if self.aid is None:
            raise BaseException('缺少必要的参数')

        f.print_1('正在获取视频信息...', end='')
        param = {
            'aid': str(self.aid)
        }
        http_result = requests.get(v.URL_UP_ALBUM, params=param,
                                   headers=f.new_http_header(v.URL_UP_ALBUM))
        if http_result.status_code == 200:
            f.print_g('OK {}'.format(http_result.status_code))
        else:
            f.print_r('RE {}'.format(http_result.status_code))
        json_data = json.loads(http_result.text)
        if json_data['code'] != 0:
            raise BaseException('获取数据的过程发生错误')

        # 修改对象信息
        self.aid = json_data['data']['aid']
        self.time = json_data['data']['ctime']
        self.desc = json_data['data']['desc']
        self.name = json_data['data']['title']
        self.zone = json_data['data']['tname']
        self.num = json_data['data']['videos']
        self.cover = json_data['data']['pic']
        self.like = json_data['data']['stat']['like']
        self.coin = json_data['data']['stat']['coin']
        self.favorite = json_data['data']['stat']['favorite']
        self.share = json_data['data']['stat']['share']
        self.view = json_data['data']['stat']['view']
        self.danmu = json_data['data']['stat']['danmaku']
        self.video_list = list()

        for page in json_data['data']['pages']:
            cv = Video(self.aid, page['cid'], page['page'], page['part'])
            cv.set_cookie(self.cookie)
            self.video_list.append(cv)

        return vars(self).copy()

    def get_album_data(self, base_path='', name_path=False, max_length=None):
        if len(self.video_list) == 0:
            self.get_album_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.aid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.aid)
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        f.print_1('正在获取视频封面--', end='')
        f.print_b('av:{}'.format(self.aid))
        http_result = requests.get(self.cover)
        with open(cache_path + '/cover.jpg', 'wb') as file:
            file.write(http_result.content)
        f.print_g('[OK]', end='')
        f.print_1('视频封面已保存')

        for video in self.video_list:
            video.get_video_data(cache_path, name_path, max_length)

        with open(cache_path + '/info.json', 'w', encoding='utf8') as file:
            file.write(str(json.dumps(self.get_dict_info())))

    def is_exist(self, base_path='', name_path=False):
        if self.aid is None:
            raise BaseException('缺少必要的参数')
        if self.name is None:
            self.get_album_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.aid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.aid)

        if os.path.exists(cache_path):
            return True
        else:
            return False

    def get_dict_info(self):
        json_data = vars(self).copy()
        video_list = []
        if 'video_list' in json_data:
            for video in json_data['video_list']:
                video_list.append(video.get_dict_info())
            json_data['video_list'] = video_list
        return json_data



import os
import re
_aria2c_result = os.popen('aria2c -v').read()
_aria2c_result = re.match(r'(\S+)', _aria2c_result)
if _aria2c_result is not None and _aria2c_result.group(1) == 'aria2':
    print('\033[0;32;0m{}\033[0m'.format(str('(=・ω・=)aria2c环境检查通过(=・ω・=)')))
    from .bili_album import Album
    from .bili_channel import Channel
    from .bili_user import User
else:
    print('\033[0;31;0m{}\033[0m'.format(str('(=・ω・=)您未配置aria2c下载环境(=・ω・=)')))

_ffmpeg_result = os.popen('ffmpeg -version').read()
_ffmpeg_result = re.match(r'(\S+)', _ffmpeg_result)
if _ffmpeg_result is not None and _ffmpeg_result.group(1) == 'ffmpeg':
    print('\033[0;32;0m{}\033[0m'.format(str('(=・ω・=)ffmpeg环境检查通过(=・ω・=)')))
    from .static_func import merge_video_file
else:
    print('\033[0;31;0m{}\033[0m'.format(str('(=・ω・=)您未配置ffmpeg渲染环境(=・ω・=)')))



HTTP_HEADER = {
    'Host': '',
    'Connection': 'keep-alive',
    'Accept': 'application/json, text/plain, */*',
    'User-Agent': '',
    'Referer': 'https://www.bilibili.com',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7'
}

URL_UP_ALL_VIDEO = 'http://space.bilibili.com/ajax/member/getSubmitVideos'
URL_UP_CHANNEL = 'http://api.bilibili.com/x/space/channel/video'
URL_UP_ALBUM = 'http://api.bilibili.com/x/web-interface/view'
URL_UP_VIDEO = 'http://api.bilibili.com/x/player/playurl'
URL_UP_INFO = 'http://api.bilibili.com/x/space/acc/info'
URL_UP_CHANNELS = 'http://api.bilibili.com/x/space/channel/list'







import os
import subprocess
from urllib import parse
import BiliUtil.static_value as v
from fake_useragent import UserAgent


def print_0(message, end='\n'):
    print('\033[0;30;0m{}\033[0m'.format(str(message)), end=end)


def print_1(message, end='\n'):
    print('\033[0;37;0m{}\033[0m'.format(str(message)), end=end)


def print_r(message, end='\n'):
    print('\033[0;31;0m{}\033[0m'.format(str(message)), end=end)


def print_g(message, end='\n'):
    print('\033[0;32;0m{}\033[0m'.format(str(message)), end=end)


def print_y(message, end='\n'):
    print('\033[0;33;0m{}\033[0m'.format(str(message)), end=end)


def print_b(message, end='\n'):
    print('\033[0;34;0m{}\033[0m'.format(str(message)), end=end)


def print_cyan(message, end='\n'):
    # 青色
    print('\033[0;36;0m{}\033[0m'.format(str(message)), end=end)


def print_gray(message, end='\n'):
    # 灰色
    print('\033[0;37;0m{}\033[0m'.format(str(message)), end=end)


def new_http_header(url):
    header = v.HTTP_HEADER.copy()
    header['Host'] = parse.urlparse(url).netloc
    header['User-Agent'] = UserAgent().random
    return header


def merge_video_file(path, delete=False):
    # 完成目录下flv视频文件与aac音频文件的合并
    if os.path.exists(path) and os.path.isdir(path):
        path = os.path.abspath(path)  # 将路径替换为绝对路径
        file_list = os.listdir(path)  # 获取路径下文件列表
        for file in file_list:
            file_path = path + '\\' + file
            file_name = os.path.splitext(file)[0]
            prefix, suffix = os.path.splitext(file_path)
            if suffix == '.flv':
                if os.path.exists(prefix + '.aac'):
                    print_b('已找到未合并文件{}，正在合并'.format(file_name))
                    shell = 'ffmpeg -i "{}.flv" -i "{}.aac" -c copy -f mp4 -y "{}.mp4"'
                    process = subprocess.Popen(shell.format(prefix, prefix, prefix),
                                               stdout=subprocess.PIPE,
                                               stderr=subprocess.PIPE)
                    process.communicate()
                    if os.path.exists(prefix + '.mp4'):
                        print_1('视频', end='')
                        print_cyan(file_name, end='')
                        print_1('已经', end='')
                        print_g('完成合并', end='')
                        if delete:
                            os.remove(prefix + '.flv')
                            os.remove(prefix + '.aac')
                            print_1('，原始音视频', end='')
                            print_r('已删除')
                        else:
                            print_1('，原始音视频', end='')
                            print_y('未删除')
                    else:
                        raise BaseException('视频与音频合并失败，不知道发生了什么')
                else:
                    raise BaseException('未找到与视频匹配的音频!!!')
            elif suffix == '.mp4':
                print_1('视频', end='')
                print_cyan(file_name, end='')
                print_1('已找到，跳过该文件')
    else:
        raise BaseException('输入的文件夹路径不存在')



import os
import re
import json
import requests
from urllib import parse

import BiliUtil.static_value as v
import BiliUtil.static_func as f
from BiliUtil.bili_album import Album


class Channel:
    cookie = None

    uid = None
    cid = None

    name = None
    cover = None
    count = None
    album_list = list()

    def __init__(self, uid=None, cid=None):
        self.set_channel(uid, cid)

    def set_channel(self, uid, cid):
        self.uid = uid
        self.cid = cid
        self.name = None
        self.cover = None
        self.count = None
        self.album_list = list()

    def set_by_url(self, url):
        input_url = parse.urlparse(url)
        uid = re.match('/([0-9]+)/channel/detail', input_url.path).group(1)
        cid = parse.parse_qs(input_url.query)['cid'][0]
        self.set_channel(uid, cid)

    def set_cookie(self, cookie):
        self.cookie = cookie
        for album in self.album_list:
            album.set_cookie(cookie)

    def get_channel_info(self):
        if self.uid is None or self.cid is None:
            raise BaseException('缺少必要的参数')

        param = {
            'mid': str(self.uid),
            'cid': str(self.cid),
            'pn': 1,  # 当前页码下标
            'ps': 30,  # 每页视频数量
            'order': 0  # 默认排序
        }
        while True:
            f.print_1('正在获取视频列表-{}...'.format(param['pn']), end='')
            http_result = requests.get(v.URL_UP_CHANNEL, params=param,
                                       headers=f.new_http_header(v.URL_UP_CHANNEL))
            if http_result.status_code == 200:
                f.print_g('OK {}'.format(http_result.status_code))
            else:
                f.print_r('RE {}'.format(http_result.status_code))
            json_data = json.loads(http_result.text)
            if json_data['code'] != 0:
                raise BaseException('获取数据的过程发生错误')

            self.name = json_data['data']['list']['name']
            self.cover = json_data['data']['list']['cover']
            self.count = str(json_data['data']['list']['count'])

            for album in json_data['data']['list']['archives']:
                av = Album(album['aid'])
                av.set_cookie(self.cookie)
                self.album_list.append(av)

            # 循环翻页获取并自动退出循环
            if len(self.album_list) >= int(json_data['data']['page']['count']):
                break
            else:
                param['pn'] += 1
        return vars(self).copy()

    def get_channel_data(self, base_path='', name_path=False, max_length=None, exclude_list=None):
        if len(self.album_list) == 0:
            self.get_channel_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.cid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.cid)
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        f.print_1('正在获取频道封面--', end='')
        f.print_b('channel:{}'.format(self.name))
        http_result = requests.get(self.cover)
        with open(cache_path + '/cover.jpg', 'wb') as file:
            file.write(http_result.content)
        f.print_g('[OK]', end='')
        f.print_1('视频封面已保存')

        for album in self.album_list:
            if exclude_list is not None and album.aid in exclude_list:
                continue
            album.get_album_data(cache_path, name_path, max_length)

        with open(cache_path + '/info.json', 'w', encoding='utf8') as file:
            file.write(str(json.dumps(self.get_dict_info())))

    def get_exist_list(self, base_path='', name_path=False):
        if len(self.album_list) == 0:
            self.get_channel_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.cid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.cid)
        if not os.path.exists(cache_path):
            return []

        exist_list = []
        for album in self.album_list:
            if album.is_exist(cache_path, name_path):
                exist_list.append(album.aid)

        return exist_list

    def get_av_list(self):
        if len(self.album_list) == 0:
            self.get_channel_info()

        av_list = []
        for album in self.album_list:
            av_list.append(album.aid)

        return av_list

    def get_dict_info(self):
        json_data = vars(self).copy()
        album_list = []
        for album in json_data['album_list']:
            album_list.append(album.get_dict_info())
        json_data['album_list'] = album_list
        return json_data



import os
import re
import json
import requests
from urllib import parse

import BiliUtil.static_value as v
import BiliUtil.static_func as f
from BiliUtil.bili_album import Album
from BiliUtil.bili_channel import Channel


class User:
    cookie = None

    uid = None

    name = None
    birthday = None
    coin = None
    face = None
    time = None
    level = None
    sex = None
    sign = None

    album_list = list()
    channel_list = list()

    def __init__(self, uid=None):
        self.uid = uid

    def set_user(self, uid):
        self.uid = uid
        self.name = None
        self.birthday = None
        self.coin = None
        self.face = None
        self.time = None
        self.level = None
        self.sex = None
        self.sign = None
        self.album_list = list()
        self.channel_list = list()

    def set_by_url(self, url):
        input_url = parse.urlparse(url)
        uid = re.match('/([0-9]+)', input_url.path).group(1)
        self.set_user(uid)

    def set_cookie(self, cookie):
        self.cookie = cookie
        for channel in self.channel_list:
            channel.set_cookie(cookie)
        for video in self.album_list:
            video.set_cookie(cookie)

    def get_user_info(self):
        if self.uid is None:
            raise BaseException('缺少必要的参数')

        f.print_1('正在获取用户信息...', end='')
        param = {
            'mid': str(self.uid),
            'jsonp': 'jsonp'
        }
        http_result = requests.get(url=v.URL_UP_INFO, params=param,
                                   headers=f.new_http_header(v.URL_UP_INFO))
        if http_result.status_code == 200:
            f.print_g('OK {}'.format(http_result.status_code))
        else:
            f.print_r('RE {}'.format(http_result.status_code))
        json_data = json.loads(http_result.text)
        if json_data['code'] != 0:
            raise BaseException('获取数据的过程发生错误')

        # 修改对象信息
        self.uid = json_data['data']['mid']
        self.name = json_data['data']['name']
        self.birthday = json_data['data']['birthday']
        self.coin = json_data['data']['coins']
        self.face = json_data['data']['face']
        self.time = json_data['data']['jointime']
        self.level = json_data['data']['level']
        self.sex = json_data['data']['sex']
        self.sign = json_data['data']['sign']

        return vars(self).copy()

    def get_channel_video_info(self):
        # 获取UP所有频道的所有视频
        if self.uid is None:
            raise BaseException('缺少必要的参数')

        if self.name is None:
            self.get_user_info()

        f.print_1('正在获取频道列表...', end='')
        param = {
            'mid': str(self.uid),
            'guest': False,
            'jsonp': 'jsonp'
        }
        http_result = requests.get(v.URL_UP_CHANNELS, params=param,
                                   headers=f.new_http_header(v.URL_UP_CHANNELS))
        if http_result.status_code == 200:
            f.print_g('OK {}'.format(http_result.status_code))
        else:
            f.print_r('RE {}'.format(http_result.status_code))
        json_data = json.loads(http_result.text)
        if json_data['code'] != 0:
            raise BaseException('获取数据的过程发生错误')

        for channel in json_data['data']['list']:
            ch = Channel(self.uid, channel['cid'])
            ch.set_cookie(self.cookie)
            self.channel_list.append(ch)

        return self.channel_list.copy()

    def get_channel_video_data(self, base_path='', name_path=False, max_length=None, exclude_list=None):
        if len(self.channel_list) == 0:
            self.get_channel_video_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.uid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.uid)
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        f.print_1('正在获取用户头像--', end='')
        f.print_b('user:{}'.format(self.name))
        http_result = requests.get(self.face)
        with open(cache_path + '/face.jpg', 'wb') as file:
            file.write(http_result.content)
        f.print_g('[OK]', end='')
        f.print_1('用户头像已保存')

        for channel in self.channel_list:
            channel.get_channel_data(cache_path, name_path, max_length, exclude_list)

        with open(cache_path + '/info.json', 'w', encoding='utf8') as file:
            file.write(str(json.dumps(self.get_dict_info())))

    def get_channel_exist_list(self, base_path='', name_path=False):
        if len(self.channel_list) == 0:
            self.get_channel_video_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.uid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.uid)
        if not os.path.exists(cache_path):
            return []

        exist_list = []
        for channel in self.channel_list:
            exist_list.extend(channel.get_exist_list(cache_path, name_path))

        return exist_list

    def get_all_video_info(self):
        # 获取UP主的所有视频的列表信息
        if self.uid is None:
            raise BaseException('缺少必要的参数')

        if self.name is None:
            self.get_user_info()

        f.print_1('正在获取视频列表...', end='')
        param = {
            'mid': str(self.uid),
            'pagesize': 30,
            'tid': 0,
            'page': 1,
            'order': 'pubdate'
        }
        while True:
            http_result = requests.get(v.URL_UP_ALL_VIDEO, params=param,
                                       headers=f.new_http_header(v.URL_UP_ALL_VIDEO))
            if http_result.status_code == 200:
                f.print_g('OK {}'.format(http_result.status_code))
            else:
                f.print_r('RE {}'.format(http_result.status_code))
            json_data = json.loads(http_result.text)
            if json_data['status'] is not True:
                raise BaseException('获取数据的过程发生错误')

            for video in json_data['data']['vlist']:
                av = Album(video['aid'])
                av.set_cookie(self.cookie)
                self.album_list.append(av)

            # 循环翻页获取并自动退出循环
            if len(self.album_list) >= int(json_data['data']['pages']):
                break
            else:
                param['pn'] += 1

        return self.album_list.copy()

    def get_all_video_data(self, base_path='', name_path=False, max_length=None, exclude_list=None):
        if len(self.album_list) == 0:
            self.get_all_video_info()

        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.uid
            cache_path = base_path + './{}'.format(temp_name)
        else:
            cache_path = base_path + './{}'.format(self.uid)
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        f.print_1('正在获取用户头像--', end='')
        f.print_b('user:{}'.format(self.name))
        http_result = requests.get(self.face)
        with open(cache_path + '/face.jpg', 'wb') as file:
            file.write(http_result.content)
        f.print_g('[OK]', end='')
        f.print_1('用户头像已保存')

        for album in self.album_list:
            if exclude_list is not None and album.aid in exclude_list:
                continue
            album.get_album_data(cache_path, name_path, max_length)

        with open(cache_path + '/info.json', 'w', encoding='utf8') as file:
            file.write(str(json.dumps(self.get_dict_info())))

    def get_all_video_exist_list(self, base_path='', name_path=False):
        if len(self.album_list) == 0:
            self.get_all_video_info()

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            if len(temp_name) == 0:
                temp_name = self.cid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.cid)

        if not os.path.exists(cache_path):
            return []

        exist_list = []
        for album in self.album_list:
            if album.is_exist(cache_path, name_path):
                exist_list.append(album.aid)

        return exist_list

    def get_all_video_av_list(self):
        if len(self.album_list) == 0:
            self.get_all_video_info()

        av_list = []
        for album in self.album_list:
            av_list.append(album.aid)

        return av_list

    def get_dict_info(self):
        json_data = vars(self).copy()

        album_list = []
        for album in json_data['album_list']:
            album_list.append(album.get_dict_info())
        json_data['album_list'] = album_list

        channel_list = []
        for channel in json_data['channel_list']:
            channel_list.append(channel.get_dict_info())
        json_data['channel_list'] = channel_list

        return json_data



import os
import re
import json
import requests
import subprocess

import BiliUtil.static_value as v
import BiliUtil.static_func as f


class Video:
    cookie = None

    aid = None
    cid = None
    index = None  # Page index 分P下标
    name = None  # Page name 分P名称

    quality = None
    quality_des = None
    length = None
    video = None
    audio = None

    def __init__(self, aid=None, cid=None, index=None, name=None):
        self.aid = aid
        self.cid = cid
        self.index = index
        self.name = name

    def set_video(self, aid=None, cid=None, index=None, name=None):
        self.aid = aid
        self.cid = cid
        self.index = index
        self.name = name
        self.quality = None
        self.length = None
        self.video = None
        self.audio = None

    def set_cookie(self, cookie):
        if isinstance(cookie, dict):
            self.cookie = {
                'SESSDATA': cookie['SESSDATA']
            }
        elif isinstance(cookie, str) and len(cookie) > 0:
            for line in cookie.split(';'):
                name, value = line.strip().split('=', 1)
                if name == 'SESSDATA':
                    self.cookie = {
                        'SESSDATA': value
                    }
                    break
        else:
            self.cookie = dict()

    def get_video_info(self, qn=116):
        if self.aid is None or self.cid is None:
            raise BaseException('缺少必要的参数')

        f.print_1('正在获取分P信息...', end='')
        param = {
            'avid': str(self.aid),
            'cid': str(self.cid),
            'qn': qn,  # 默认使用最高画质下载
            'otype': 'json',
            'fnver': 0,
            'fnval': 16
        }
        http_result = requests.get(v.URL_UP_VIDEO, params=param, cookies=self.cookie,
                                   headers=f.new_http_header(v.URL_UP_INFO))
        if http_result.status_code == 200:
            f.print_g('OK {}'.format(http_result.status_code))
        else:
            f.print_r('RE {}'.format(http_result.status_code))
        json_data = json.loads(http_result.text)
        if json_data['code'] != 0:
            raise BaseException('获取数据的过程发生错误')

        # 自动识别不同的数据来源
        if 'dash' in json_data['data']:
            self.quality = json_data['data']['quality']
            self.length = json_data['data']['timelength']
            self.video = json_data['data']['dash']['video'][-1]['baseUrl']
            self.audio = json_data['data']['dash']['audio'][0]['baseUrl']
        elif 'durl' in json_data['data']:
            self.quality = json_data['data']['quality']
            self.length = json_data['data']['timelength']
            self.video = json_data['data']['durl'][-1]['url']

        for index, val in enumerate(json_data['data']['accept_quality']):
            if val == self.quality:
                self.quality_des = json_data['data']['accept_description'][index]
                break

        return self

    def get_video_data(self, base_path='', name_path=False, max_length=None):
        if self.video is None and self.audio is None:
            self.get_video_info()

        if max_length is not None and max_length < self.length:
            f.print_y('视频：{}，超出限定长度取消下载')
            return

        base_path = os.path.abspath(base_path)  # 获取绝对路径地址
        if name_path:
            # 检查路径名中的特殊字符
            temp_name = re.sub(r"[\/\\\:\*\?\"\<\>\|\s'‘’]", '_', self.name)
            temp_name = re.sub(r'[‘’]', '_', temp_name)
            if len(temp_name) == 0:
                temp_name = self.cid
            cache_path = base_path + '/{}'.format(temp_name)
        else:
            cache_path = base_path + '/{}'.format(self.cid)
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        # 使用两个进程分别下载视频和音频
        f.print_1('正在下载视频和配套音--', end='')
        f.print_b('av:{},cv:{}'.format(self.aid, self.cid))
        if self.audio is not None and self.video is not None:
            self.aria2c_download(cache_path, '{}_{}.aac'.format(self.cid, self.quality_des), self.audio)
            self.aria2c_download(cache_path, '{}_{}.flv'.format(self.cid, self.quality_des), self.video)
        if self.video is not None and self.audio is None:
            self.aria2c_download(cache_path, '{}_{}.mp4'.format(self.cid, self.quality_des), self.video)
        else:
            f.print_y('无需独立下载音频')
        f.print_cyan('==============================================================')

        with open(cache_path + '/info.json', 'w', encoding='utf8') as file:
            file.write(str(json.dumps(self.get_dict_info())))

    def aria2c_download(self, cache_path, file_name, download_url):
        referer = 'https://www.bilibili.com/video/av' + str(self.aid)
        shell = 'aria2c -c -s 1 -d "{}" -o "{}" --referer="{}" "{}"'
        shell = shell.format(cache_path, file_name, referer, download_url)
        process = subprocess.Popen(shell)
        process.wait()

        file_path = '{}/{}'.format(cache_path, file_name)
        if os.path.exists(file_path):
            f.print_g('[OK]', end='')
            f.print_1('文件{}下载成功--'.format(file_name), end='')
            f.print_b('av:{},cv:{}'.format(self.aid, self.cid))
        else:
            f.print_r('[ERR]', end='')
            f.print_1('文件{}下载失败--'.format(file_name), end='')
            f.print_b('av:{},cv:{}'.format(self.aid, self.cid))
            f.print_r(shell.format(file_path, referer, download_url))
            raise BaseException('av:{},cv:{},下载失败'.format(self.aid, self.cid))

    def get_dict_info(self):
        json_data = vars(self).copy()
        return json_data
